#!/usr/bin/octave

DIM_MAX       =  5;
LEN_MAX       =  7;
VAL_MIN       = -9;
VAL_MAX       =  9;

P_LEN_NONZERO =  0.95;
P_BSX         =  0.05;
P_SINGLE      =  0.5;
P_SCALAR      =  0.3;
P_NAN         =  0.15;
P_REAL        =  0.5;
P_SPARSE_ZERO =  0.3;

gen_real      = @(dv) randi([VAL_MIN VAL_MAX], dv) + ...
                      ifelse(rand(dv) < P_NAN,
                             ifelse(rand(dv) < 0.5, NaN, NA),
                             0);
gen_complex   = @(dv) gen_real(dv) + I * gen_real(dv);

num_tests = str2num(argv(){2});

for t = 1 : num_tests
    printf('Generating test %d ... ', t);
    fflush(stdout);

    % full inputs
    dv = randi([1 LEN_MAX], randi([1 DIM_MAX]), 1);
    nonzeros = rand(size(dv)) < P_LEN_NONZERO;
    real = rand() < P_REAL;
    float = rand() < P_SINGLE;

    for i = 1 : 2
        dv_i = ifelse(rand(size(dv)) < P_BSX, ones(size(dv)), dv);
        dv_i .*= nonzeros;
        T{t, i} = ifelse(real,
                         gen_real(dv_i),
                         gen_complex(dv_i));
        if float
            T{t, i} = single(T{t, i});
        end;
    end;

    % sparse inputs
    m = randi([0 LEN_MAX]);
    n = randi([0 LEN_MAX]);
    real = rand() < P_REAL;

    scalar = [false false];
    if (rand() < P_SCALAR)
        scalar(randi([1 2])) = true;
    end;

    for i = 1 : 2
        if scalar(i)
            T{t, i + 2} = ifelse(real, gen_real(1), gen_complex(1));
        else
            temp = ifelse(real,
                          gen_real([m n]),
                          gen_complex([m n]));

            temp = ifelse(rand(m, n) < P_SPARSE_ZERO,
                          0,
                          temp);

            T{t, i + 2} = sparse(temp);
        end;
    end;

    disp('done.');
    fflush(stdout);
end;

save(argv(){1}, 'T');
